@@ -8,8 +8,10 @@
 # Copyright (C) 2007       Brian G. Matherly
 # Copyright (C) 2009       Benny Malengier
 # Copyright (C) 2009       Gary Burton
+# Copyright (C) 2017       Jonathan Biegert <azrdev@qrdn.de>
 # Copyright (C) 2017       Mindaugas Baranauskas
 # Copyright (C) 2017       Paul Culley
+# Copyright (C) 2018       Christophe aka khrys63
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -63,7 +65,7 @@ LOG = logging.getLogger(".graphdoc")
 # Private Constants
 #
 #-------------------------------------------------------------------------
-_FONTS = [{'name' : _("Default"), 'value' : ""},
+_FONTS = [{'name' : _("Default"), 'value' : "serif"},
           {'name' : _("PostScript / Helvetica"), 'value' : "Helvetica"},
           {'name' : _("TrueType / FreeSans"), 'value' : "FreeSans"}]
 
@@ -72,6 +74,11 @@ _RANKDIR = [{'name' : _("Vertical (↓)"), 'value'   : "TB"},
             {'name' : _("Horizontal (→)"), 'value' : "LR"},
             {'name' : _("Horizontal (←)"), 'value' : "RL"}]
 
+_NODE_PORTS = {"TB" : ("n", "s"),
+               "BT" : ("s", "n"),
+               "LR" : ("w", "e"),
+               "RL" : ("e", "w")}
+
 _PAGEDIR = [{'name' : _("Bottom, left"), 'value'  : "BL"},
             {'name' : _("Bottom, right"), 'value' : "BR"},
             {'name' : _("Top, left"), 'value'     : "TL"},
@@ -183,6 +190,10 @@ class GVOptions:
         spline.set_help(_("How the lines between objects will be drawn."))
         menu.add_option(category, "spline", spline)
 
+        node_ports = BooleanOption(_("Alternate line attachment"), False)
+        node_ports.set_help(_("Whether lines attach to nodes differently"))
+        menu.add_option(category, "node_ports", node_ports)
+
         # the page direction option only makes sense when the
         # number of horizontal and/or vertical pages is > 1,
         # so we need to remember these 3 controls for later
@@ -354,6 +365,34 @@ class GVDoc(metaclass=ABCMeta):
         :return: nothing
         """
 
+    @abstractmethod
+    def add_samerank(self, id1, id2):
+        """
+        Force the same line for two linked nodes.
+
+        :param id1: The unique identifier of the starting node.
+            Example: "p55"
+        :type id1: string
+        :param id2: The unique identifier of the ending node.
+            Example: "p55"
+        :type id2: string
+        :return: nothing
+        """
+
+    @abstractmethod
+    def rewrite_label(self, id, label):
+        """
+        Rewrite the node label.
+
+        :param id: The unique identifier of the node.
+            Example: "p55"
+        :type id: string
+        :param label: The text to be displayed in the node.
+            Example: "John Smith"
+        :type label: string
+        :return: nothing
+        """
+
     @abstractmethod
     def start_subgraph(self, graph_id):
         """
@@ -409,6 +448,7 @@ class GVDocBase(BaseDoc, GVDoc):
         self.vpages = get_option('v_pages').get_value()
         self.usesubgraphs = get_option('usesubgraphs').get_value()
         self.spline = get_option('spline').get_value()
+        self.node_ports = get_option('node_ports').get_value()
 
         paper_size = paper_style.get_size()
 
@@ -456,6 +496,9 @@ class GVDocBase(BaseDoc, GVDoc):
             '  splines="%s";\n' % self.spline +
             '\n' +
             '  edge [len=0.5 style=solid fontsize=%d];\n' % self.fontsize)
+        if self.node_ports:
+            self.write('  edge [headport=%s tailport=%s];\n'
+                           % _NODE_PORTS[self.rankdir])
 
         if self.fontfamily:
             self.write('  node [style=filled fontname="%s" fontsize=%d];\n'
@@ -586,6 +629,22 @@ class GVDocBase(BaseDoc, GVDoc):
             else:
                 self.write('# %s\n' % text)
 
+    def add_samerank(self, id1, id2):
+        """
+        Force the same line for two linked nodes.
+
+        Implements GVDocBase.add_samerank().
+        """
+        self.write('  {rank=same "%s" "%s"}\n' % (id1, id2))
+
+    def rewrite_label(self, id, label):
+        """
+        Rewrite the node label.
+
+        Implements GVDocBase.rewrite_label().
+        """
+        self.write('  "%s" [label = "%s"]\n' % (id, label))
+
     def start_subgraph(self, graph_id):
         """ Implement GVDocBase.start_subgraph() """
         graph_id = graph_id.replace(' ', '_')  # for user-defined ID with space
@@ -977,8 +1036,9 @@ class GVPdfGsDoc(GVDocBase):
             tmp_pdf_piece = "%s_%d_%d.pdf" % (tmp_ps, __x, __y)
             list_of_pieces.append(tmp_pdf_piece)
             # Generate Ghostscript code
-            command = '%s -q -dBATCH -dNOPAUSE -dSAFER -g%dx%d '\
-                '-sOutputFile="%s" -r72 -sDEVICE=pdfwrite '\
+            command = '%s -q -dBATCH -dNOPAUSE -dSAFER '\
+                '-dDEVICEWIDTHPOINTS=%d -dDEVICEHEIGHTPOINTS=%d '\
+                '-sOutputFile="%s" -sDEVICE=pdfwrite '\
                 '-c "<</.HWMargins [%d %d %d %d] /PageOffset [%d %d]>> '\
                 'setpagedevice" -f "%s"' % (
                     _GS_CMD, width_pt + 10, height_pt + 10, tmp_pdf_piece,
@@ -988,7 +1048,7 @@ class GVPdfGsDoc(GVDocBase):
             os.system(command)
         # Merge pieces to single multipage PDF ;
         command = '%s -q -dBATCH -dNOPAUSE '\
-            '-sOUTPUTFILE="%s" -r72 -sDEVICE=pdfwrite %s '\
+            '-sOUTPUTFILE="%s" -sDEVICE=pdfwrite %s '\
             % (_GS_CMD, self._filename, ' '.join(list_of_pieces))
         os.system(command)
 
